@@ -20,22 +20,41 @@
  
 #include "scanning.h"
 
-void start_scan(char * iface, APList * list)
+int start_scan(char * iface, APList * list)
 {
-	int skfd;
+	int skfd, retval;
 	
 	skfd=iw_sockets_open();
-	print_scanning_info(skfd,iface,list);
+	retval = print_scanning_info(skfd,iface,list);
 	close(skfd);
+	return retval;
 }
 
-//Modified to support APList class. Also uses sdtout to print the scan result
-int print_scanning_info(int	skfd, char *	ifname, APList * listAPs)
+/* Modified to support APList class. Also uses sdtout to print the scan result */
+int print_scanning_info(int skfd, 
+			char * ifname, 
+			APList * listAPs)
 {
   struct iwreq		wrq;
-  unsigned char		buffer[IW_SCAN_MAX_DATA];	/* Results */
+  unsigned char *       buffer = NULL;          /* Results */
+  int                   buflen = IW_SCAN_MAX_DATA; /* Min for compat WE<17 */
+  struct iw_range       range;
+  int			has_range;
+
   struct timeval	tv;				/* Select timeout */
-  int			timeout = 6000000;		/* 5s */
+  int			timeout = 5000000;		/* 5s */
+
+  /* Get range stuff */
+  has_range = (iw_get_range_info(skfd, ifname, &range) >= 0);
+
+  /* Check if the interface could support scanning. */
+  if((!has_range) || (range.we_version_compiled < 14))
+    {
+      fprintf(stderr, "%-8.16s  Interface doesn't support scanning.\n\n",
+              ifname);
+      return(-1);
+    }
+
 
   /* Init timeout value -> 250ms*/
   tv.tv_sec = 0;
@@ -45,8 +64,9 @@ int print_scanning_info(int	skfd, char *	ifname, APList * listAPs)
    * Here we should look at the command line args and set the IW_SCAN_ flags
    * properly
    */
-  wrq.u.param.flags = IW_SCAN_DEFAULT;
-  wrq.u.param.value = 0;		/* Later */
+  wrq.u.data.pointer = NULL;            /* Later */
+  wrq.u.data.flags = 0;
+  wrq.u.data.length = 0;
 
   /* Initiate Scanning */
   if(iw_set_ext(skfd, ifname, SIOCSIWSCAN, &wrq) < 0)
@@ -55,16 +75,16 @@ int print_scanning_info(int	skfd, char *	ifname, APList * listAPs)
 	{
 	  fprintf(stderr, "%-8.8s  Interface doesn't support scanning : %s\n\n",
 		  ifname, strerror(errno));
-	  return(0);
+	  return(-1);
 	}
-      /* If we don't have the permission to initiate the scan, we may
-       * still have permission to read left-over results.
-       * But, don't wait !!! */
 #if 0
       /* Not cool, it display for non wireless interfaces... */
       fprintf(stderr, "%-8.8s  (Could not trigger scanning, just reading left-over results)\n", ifname);
 #endif
-      tv.tv_usec = 0;
+      /* If we don't have the permission to initiate the scan, we may
+       * still have permission to read left-over results.
+       * But, don't wait !!! */
+       tv.tv_usec = 0;
     }
   timeout -= tv.tv_usec;
 
@@ -96,12 +116,46 @@ int print_scanning_info(int	skfd, char *	ifname, APList * listAPs)
       /* Check if there was a timeout */
       if(ret == 0)
 	{
+        unsigned char *       newbuf;
+        realloc:
+          /* (Re)allocate the buffer - realloc(NULL, len) == malloc(len) */
+          newbuf = (unsigned char *)realloc(buffer, buflen);
+          if(newbuf == NULL)
+            {
+              if(buffer)
+                free(buffer);
+              fprintf(stderr, "%s: Allocation failed\n", __FUNCTION__);
+              return(-2);
+            }
+          buffer = newbuf;
+
 	  /* Try to read the results */
-	  wrq.u.data.pointer = (char *)buffer;
+	  wrq.u.data.pointer = buffer;
 	  wrq.u.data.flags = 0;
-	  wrq.u.data.length = sizeof(buffer);
+	  wrq.u.data.length = buflen;
 	  if(iw_get_ext(skfd, ifname, SIOCGIWSCAN, &wrq) < 0)
 	    {
+              /* Check if buffer was too small (WE-17 only) */
+              if((errno == E2BIG) && (range.we_version_compiled > 16))
+                {
+                  /* Some driver may return very large scan results, either
+                   * because there are many cells, or because they have many
+                   * large elements in cells (like IWEVCUSTOM). Most will
+                   * only need the regular sized buffer. We now use a dynamic
+                   * allocation of the buffer to satisfy everybody. Of course,
+                   * as we don't know in advance the size of the array, we try
+                   * various increasing sizes. Jean II */
+
+                  /* Check if the driver gave us any hints. */
+                  if(wrq.u.data.length > buflen)
+                    buflen = wrq.u.data.length;
+                  else
+                    buflen *= 2;
+
+                  /* Try again */
+                  goto realloc;
+                }
+
 	      /* Check if results not available yet */
 	      if(errno == EAGAIN)
 		{
@@ -114,9 +168,10 @@ int print_scanning_info(int	skfd, char *	ifname, APList * listAPs)
 		}
 
 	      /* Bad error */
+	      free(buffer);
 	      fprintf(stderr, "%-8.8s  Failed to read scan data : %s\n\n",
 		      ifname, strerror(errno));
-	      return(1);
+	      return(-3);
 	    }
 	  else
 	    /* We have the results, go to process them */
@@ -133,11 +188,8 @@ int print_scanning_info(int	skfd, char *	ifname, APList * listAPs)
       struct stream_descr	stream;
       int			ap_num = 1;
       int			ret;
-      struct iw_range		range;
-      int			has_range;
       APEntry* 			NewAP = NULL; 
-      
-      //int we_version = iw_get_kernel_we_version();
+ 
 #if 0
       /* Debugging code. In theory useless, because it's debugged ;-) */
       int	i;
@@ -146,37 +198,34 @@ int print_scanning_info(int	skfd, char *	ifname, APList * listAPs)
 	printf(":%02X", buffer[i]);
       printf("]\n");
 #endif
-      has_range = (iw_get_range_info(skfd, ifname, &range) >= 0);
       printf("%-8.8s  Scan completed :\n", ifname);
       iw_init_event_stream(&stream, (char *)buffer, wrq.u.data.length);
       do
-	{
+      {
 	  /* Extract an event and print it */
-	  ret = iw_extract_event_stream(&stream, &iwe);
-	  if(ret > 0)
-	  {
-	    	//ap_num = print_scanning_token(&iwe, ap_num, &range, has_range);
-		  char buffer[128];	/* Temporary buffer */
+	  ret = iw_extract_event_stream(&stream, &iwe, range.we_version_compiled);
+	if(ret > 0)
+	{
+		char buffer[128];	/* Temporary buffer */
 		
-		  /* Now, let's decode the event */
-		  switch(iwe.cmd)
-		    {
-		    case SIOCGIWAP:
-		      // printf("          Cell %02d - Address: %s\n", ap_num, iw_pr_ether(buffer, (const unsigned char *)iwe.u.ap_addr.sa_data));
-		      printf("New AP found(%02d): %s\n", ap_num, iw_pr_ether(buffer, (const unsigned char *)iwe.u.ap_addr.sa_data));
-		      // APEntry* NewAP;
-		      NewAP = new APEntry;
-		      listAPs->AddNew(NewAP);
-		      NewAP->setMAC(iw_pr_ether(buffer, (const unsigned char *)iwe.u.ap_addr.sa_data)); ap_num++;
-		      break;
-		    case SIOCGIWNWID:
-/*		      if(iwe.u.nwid.disabled)
-			printf("                    NWID:off/any\n");
-		      else
-			printf("                    NWID:%X\n", iwe.u.nwid.value);
-*/		      break;
-		    case SIOCGIWFREQ:
-		      {
+		/* Now, let's decode the event */
+		switch(iwe.cmd)
+		{
+		  case SIOCGIWAP:
+		      	printf("New AP found(%02d): %s\n", ap_num, iw_pr_ether(buffer, (const unsigned char *)iwe.u.ap_addr.sa_data));
+		      	// APEntry* NewAP;
+		      	NewAP = new APEntry;
+		      	listAPs->append(NewAP);
+		      	NewAP->setMAC(iw_pr_ether(buffer, (const unsigned char *)iwe.u.ap_addr.sa_data)); ap_num++;
+		      	break;
+		  case SIOCGIWNWID:
+/*			if(iwe.u.nwid.disabled)
+				printf("                    NWID:off/any\n");
+			else
+				printf("                    NWID:%X\n", iwe.u.nwid.value);
+*/			break;
+		  case SIOCGIWFREQ:
+		  {
 			double		freq;			/* Frequency/channel */
 
 			freq = iw_freq2float(&(iwe.u.freq));
@@ -184,112 +233,110 @@ int print_scanning_info(int	skfd, char *	ifname, APList * listAPs)
 				NewAP->setChannel((unsigned int)iw_freq_to_channel(freq,&range));
 			else
 				NewAP->setChannel((unsigned int)freq);
-			#if DEBUG_STDOUT
-				iw_print_freq(buffer, freq);
+			#if SWSDEBUG
+				iw_print_freq(buffer, sizeof(buffer), freq, NewAP->getChannel(), iwe.u.freq.flags);
 				printf("                    %s\n", buffer);
 				// printf("                    Freq(Chan):%f(%d)\n",freq, iw_freq_to_channel(freq,&range));
 			#endif
-		      }
-		      break;
-		    case SIOCGIWMODE:
-			#if DEBUG_STDOUT
+		  }
+		  break;
+		  case SIOCGIWMODE:
+			#if SWSDEBUG
 				printf("                    Mode:%s\n", iw_operation_mode[iwe.u.mode]);
 			#endif
-		      if (iwe.u.mode == 1) 		//ad-hoc
-		      {
-			      NewAP->setMode(1);
-		      }
-		      else if (iwe.u.mode == 3) 	//master
-		      {
-			      NewAP->setMode(0);
-		      }
-		      break;
-		    case SIOCGIWNAME:
-			#if DEBUG_STDOUT
+			if (iwe.u.mode == 1) 		//ad-hoc
+			{
+				NewAP->setMode(1);
+			}
+			else if (iwe.u.mode == 3) 	//master
+			{
+				NewAP->setMode(0);
+			}
+			break;
+		  case SIOCGIWNAME:
+			#if SWSDEBUG
 				printf("                    Protocol:%-1.16s\n", iwe.u.name);
 			#endif
-		      break;
-		    case SIOCGIWESSID:
-		      {
+			break;
+		  case SIOCGIWESSID:
+		  {
 			char essid[IW_ESSID_MAX_SIZE+1];
 			if((iwe.u.essid.pointer) && (iwe.u.essid.length))
-			  memcpy(essid, iwe.u.essid.pointer, iwe.u.essid.length);
+			 	memcpy(essid, iwe.u.essid.pointer, iwe.u.essid.length);
 			essid[iwe.u.essid.length] = '\0';
 			if(iwe.u.essid.flags)
-			  {
-			    /* Does it have an ESSID index ? */
-			    if((iwe.u.essid.flags & IW_ENCODE_INDEX) > 1)
-			    {
-				#if DEBUG_STDOUT
-				      printf("                    ESSID:\"%s\" [%d]\n", essid,
+			{
+				/* Does it have an ESSID index ? */
+				if((iwe.u.essid.flags & IW_ENCODE_INDEX) > 1)
+				{
+					#if SWSDEBUG
+						printf("                    ESSID:\"%s\" [%d]\n", essid,
 										(iwe.u.essid.flags &IW_ENCODE_INDEX));
-				#endif
-			    }
-			    else
-			    	#if DEBUG_STDOUT
-			      		printf("                    ESSID:\"%s\"\n", essid);
-				#endif
-			        NewAP->setEssid(essid);
-			  }
+					#endif
+				}
+				else
+					#if SWSDEBUG
+						printf("                    ESSID:\"%s\"\n", essid);
+					#endif
+					NewAP->setEssid(essid);
+			}
 			else
 			{
-			   #if DEBUG_STDOUT
-			  	printf("                    ESSID:off/any\n");
-			   #endif
+				#if SWSDEBUG
+					printf("                    ESSID:off/any\n");
+				#endif
 			}
-		      }
-		      break;
-		    case SIOCGIWENCODE:
-		      {
+		  }
+		  break;
+		  case SIOCGIWENCODE:
+		  {
 			unsigned char	key[IW_ENCODING_TOKEN_MAX];
 			if(iwe.u.data.pointer)
-			  memcpy(key, iwe.u.essid.pointer, iwe.u.data.length);
+				memcpy(key, iwe.u.essid.pointer, iwe.u.data.length);
 			else
-			  iwe.u.data.flags |= IW_ENCODE_NOKEY;
-			#if DEBUG_STDOUT  
-			  printf("                    Encryption key:");
+				iwe.u.data.flags |= IW_ENCODE_NOKEY;
+			
+			#if SWSDEBUG  
+				printf("                    Encryption key:");
 			#endif
 			if(iwe.u.data.flags & IW_ENCODE_DISABLED)
 			{
-			  #if DEBUG_STDOUT 
-			    printf("off\n");
-			  #endif
-			  NewAP->setWEP(false);
-		  	}
+				#if SWSDEBUG 
+					printf("off\n");
+				#endif
+				NewAP->setWEP(false);
+			}
 			else
-			  {
-			    NewAP->setWEP(true);
-			    /* Display the key */
-			    iw_print_key(buffer, key, iwe.u.data.length,
-					 iwe.u.data.flags);
-			    #if DEBUG_STDOUT
-			    	printf("%s", buffer);
-			    
-			      /* Other info... */
-			      if((iwe.u.data.flags & IW_ENCODE_INDEX) > 1)
-			        printf(" [%d]", iwe.u.data.flags & IW_ENCODE_INDEX);
-			      if(iwe.u.data.flags & IW_ENCODE_RESTRICTED)
-			        printf("   Security mode:restricted");
-			      if(iwe.u.data.flags & IW_ENCODE_OPEN)
-			        printf("   Security mode:open");
-			      printf("\n");
-			    #endif
-			  }
-		      }
-		      break;
-		    case SIOCGIWRATE:
-		    #if DEBUG_STDOUT
-		      iw_print_bitrate(buffer, iwe.u.bitrate.value);
-		      printf("                    Bit Rate:%s\n", buffer);
-		    #endif
-		      break;
-		    case IWEVQUAL:
-		      {
+			{
+				NewAP->setWEP(true);
+				/* Display the key */
+				iw_print_key(buffer, sizeof(buffer), key, iwe.u.data.length, iwe.u.data.flags);
+				#if SWSDEBUG
+					printf("%s", buffer);
+					/* Other info... */
+					if((iwe.u.data.flags & IW_ENCODE_INDEX) > 1)
+					printf(" [%d]", iwe.u.data.flags & IW_ENCODE_INDEX);
+					if(iwe.u.data.flags & IW_ENCODE_RESTRICTED)
+					printf("   Security mode:restricted");
+					if(iwe.u.data.flags & IW_ENCODE_OPEN)
+					printf("   Security mode:open");
+					printf("\n");
+				#endif
+			}
+		  }
+		  break;
+		  case SIOCGIWRATE:
+			#if SWSDEBUG
+				iw_print_bitrate(buffer, sizeof(buffer), iwe.u.bitrate.value);
+				printf("                    Bit Rate:%s\n", buffer);
+			#endif
+			break;
+		  case IWEVQUAL:
 			iwe.u.qual.updated = 0x0;	/* Not that reliable, disable */
-			iw_print_stats(buffer, &iwe.u.qual, &range, has_range);
+			iw_print_stats(buffer, sizeof(buffer), &iwe.u.qual, &range, has_range);
 			if(has_range && (iwe.u.qual.level != 0))
 			{
-			      /* If the statistics are in dBm */
+				/* If the statistics are in dBm */
 				if(iwe.u.qual.level > range.max_qual.level)
 				{
 					NewAP->setSignal(iwe.u.qual.level - 0x100);
@@ -301,25 +348,23 @@ int print_scanning_info(int	skfd, char *	ifname, APList * listAPs)
 					NewAP->setNoise(iwe.u.qual.noise);
 				}
 			}
-
-			#if DEBUG_STDOUT
-			  printf("                    %s\n", buffer);
+			#if SWSDEBUG
+				printf("                    %s\n", buffer);
 			#endif
 			break;
-		      }
 		#if WIRELESS_EXT > 14
-		    case IWEVCUSTOM:
-		      {
+		  case IWEVCUSTOM:
+		  {
 			char custom[IW_CUSTOM_MAX+1];
 			if((iwe.u.data.pointer) && (iwe.u.data.length))
-			  memcpy(custom, iwe.u.data.pointer, iwe.u.data.length);
+				memcpy(custom, iwe.u.data.pointer, iwe.u.data.length);
 			custom[iwe.u.data.length] = '\0';
-			#if DEBUG_STDOUT
-			  printf("                    Extra:%s\n", custom);
+			#if SWSDEBUG
+				printf("                    Extra:%s\n", custom);
 			#endif
 			// For ipw driver (Intel Centrino)
 			QString extra = QString(custom).stripWhiteSpace();
-			#if DEBUG_STDOUT
+			#if SWSDEBUG
 				printf("                    (%d)(%d)\n",extra.find("Signal"),extra.find("RSSI"));
 			#endif
 			if ((extra.find("Signal") != -1) || (extra.find("RSSI") !=-1))
@@ -330,32 +375,32 @@ int print_scanning_info(int	skfd, char *	ifname, APList * listAPs)
 				QStringList::Iterator it = lst.begin();
 				it++; // point to the next word so (*it) has the signal level
 				s = QVariant(*it).toInt(&ok);
-				#if DEBUG_STDOUT
-				   if (ok)
-				      printf("                    AP Signal found in 'Extra' event: %d dBm",s);
-				   else
-				      printf("                    Unable to parse 'Extra' signal event");
+				#if SWSDEBUG
+					if (ok)
+						printf("                    AP Signal found in 'Extra' event: %d dBm",s);
+					else
+						printf("                    Unable to parse 'Extra' signal event");
 				#endif
 				NewAP->setSignal(s);
 				NewAP->setNoise(-256);	// I use this value since ipw driver doesn't support noise levels yet.
 			}
-				
-		      }
-		      break;
+		  }
+		  break;
 		#endif /* WIRELESS_EXT > 14 */
-		    default:
-		      printf("                    (Unknown Wireless Token 0x%04X)\n",
-			     iwe.cmd);
-		   }	/* switch(iwe.cmd) */
-	  }	  
+		  default:
+			printf("                    (Unknown Wireless Token 0x%04X)\n",
+			iwe.cmd);
+		}	/* switch(iwe.cmd) */
 	}
+      }
       while(ret > 0);
       printf("\n");
     }
   else
     printf("%-8.8s  No scan results\n", ifname);
 
-  return(1);
+  free(buffer);
+  return(0);
 }
 
 /*------------------------------------------------------------------*/
@@ -493,50 +538,33 @@ void GetWirelessInterfacesNames_old(char* Names, int* Number)
 	close(skfd);
 }
 
-// from iwconfig.c -->extracts DETAILED INFO
-int iw_get_info(int skfd, char * ifname, struct wireless_info * info)
+/* from iwconfig.c -->extracts DETAILED INFO */
+int iw_get_info(int			skfd,
+	 	char *			ifname,
+	 	struct wireless_info *	info)
 {
   struct iwreq		wrq;
 
   memset((char *) info, 0, sizeof(struct wireless_info));
 
-  /* Get wireless name */
-  if(iw_get_ext(skfd, ifname, SIOCGIWNAME, &wrq) < 0)
+  /* Get basic information */
+  if(iw_get_basic_config(skfd, ifname, &(info->b)) < 0)
     {
       /* If no wireless name : no wireless extensions */
       /* But let's check if the interface exists at all */
       struct ifreq ifr;
 
-      strcpy(ifr.ifr_name, ifname);
+      strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
       if(ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0)
 	return(-ENODEV);
       else
 	return(-ENOTSUP);
     }
-  else
-    {
-      strncpy(info->name, wrq.u.name, IFNAMSIZ);
-      info->name[IFNAMSIZ] = '\0';
-    }
 
   /* Get ranges */
   if(iw_get_range_info(skfd, ifname, &(info->range)) >= 0)
     info->has_range = 1;
 
-  /* Get network ID */
-  if(iw_get_ext(skfd, ifname, SIOCGIWNWID, &wrq) >= 0)
-    {
-      info->has_nwid = 1;
-      memcpy(&(info->nwid), &(wrq.u.nwid), sizeof(iwparam));
-    }
-
-  /* Get frequency / channel */
-  if(iw_get_ext(skfd, ifname, SIOCGIWFREQ, &wrq) >= 0)
-    {
-      info->has_freq = 1;
-      info->freq = iw_freq2float(&(wrq.u.freq));
-    }
-
   /* Get sensitivity */
   if(iw_get_ext(skfd, ifname, SIOCGIWSENS, &wrq) >= 0)
     {
@@ -544,27 +572,6 @@ int iw_get_info(int skfd, char * ifname, struct wireless_info * info)
       memcpy(&(info->sens), &(wrq.u.sens), sizeof(iwparam));
     }
 
-  /* Get encryption information */
-  wrq.u.data.pointer = (caddr_t) info->key;
-  wrq.u.data.length = IW_ENCODING_TOKEN_MAX;
-  wrq.u.data.flags = 0;
-  if(iw_get_ext(skfd, ifname, SIOCGIWENCODE, &wrq) >= 0)
-    {
-      info->has_key = 1;
-      info->key_size = wrq.u.data.length;
-      info->key_flags = wrq.u.data.flags;
-    }
-
-  /* Get ESSID */
-  wrq.u.essid.pointer = (caddr_t) info->essid;
-  wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
-  wrq.u.essid.flags = 0;
-  if(iw_get_ext(skfd, ifname, SIOCGIWESSID, &wrq) >= 0)
-    {
-      info->has_essid = 1;
-      info->essid_on = wrq.u.data.flags;
-    }
-
   /* Get AP address */
   if(iw_get_ext(skfd, ifname, SIOCGIWAP, &wrq) >= 0)
     {
@@ -601,14 +608,6 @@ int iw_get_info(int skfd, char * ifname, struct wireless_info * info)
       memcpy(&(info->frag), &(wrq.u.frag), sizeof(iwparam));
     }
 
-  /* Get operation mode */
-  if(iw_get_ext(skfd, ifname, SIOCGIWMODE, &wrq) >= 0)
-    {
-      info->mode = wrq.u.mode;
-      if((info->mode < IW_NUM_OPER_MODE) && (info->mode >= 0))
-	info->has_mode = 1;
-    }
-
   /* Get Power Management settings */
   wrq.u.power.flags = 0;
   if(iw_get_ext(skfd, ifname, SIOCGIWPOWER, &wrq) >= 0)
@@ -617,26 +616,29 @@ int iw_get_info(int skfd, char * ifname, struct wireless_info * info)
       memcpy(&(info->power), &(wrq.u.power), sizeof(iwparam));
     }
 
-#if WIRELESS_EXT > 9
-  /* Get Transmit Power */
-  if(iw_get_ext(skfd, ifname, SIOCGIWTXPOW, &wrq) >= 0)
+  if((info->has_range) && (info->range.we_version_compiled > 9))
     {
-      info->has_txpower = 1;
-      memcpy(&(info->txpower), &(wrq.u.txpower), sizeof(iwparam));
+      /* Get Transmit Power */
+      if(iw_get_ext(skfd, ifname, SIOCGIWTXPOW, &wrq) >= 0)
+	{
+	  info->has_txpower = 1;
+	  memcpy(&(info->txpower), &(wrq.u.txpower), sizeof(iwparam));
+	}
     }
-#endif
 
-#if WIRELESS_EXT > 10
-  /* Get retry limit/lifetime */
-  if(iw_get_ext(skfd, ifname, SIOCGIWRETRY, &wrq) >= 0)
+  if((info->has_range) && (info->range.we_version_compiled > 10))
     {
-      info->has_retry = 1;
-      memcpy(&(info->retry), &(wrq.u.retry), sizeof(iwparam));
+      /* Get retry limit/lifetime */
+      if(iw_get_ext(skfd, ifname, SIOCGIWRETRY, &wrq) >= 0)
+	{
+	  info->has_retry = 1;
+	  memcpy(&(info->retry), &(wrq.u.retry), sizeof(iwparam));
+	}
     }
-#endif	/* WIRELESS_EXT > 10 */
 
   /* Get stats */
-  if(iw_get_stats(skfd, ifname, &(info->stats)) >= 0)
+  if(iw_get_stats(skfd, ifname, &(info->stats),
+		  &info->range, info->has_range) >= 0)
     {
       info->has_stats = 1;
     }
